﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using SKBKontur.KZ.Common;
using SKBKontur.KZ.Core;
using SKBKontur.KZ.User.Helpers;

namespace SKBKontur.KZ.User.CompareSumms.AllSummCompare
{
    public static class DataHelper
    {
        public static List<SummInfo> GetData(bool selectedLs, IKZ kzApp)
        {
            var options = kzApp.GetService<IOptions>();
            var lsList = kzApp.GetService<ILsList>();
            var mtek = options.SystemMonth;
            var res = new List<SummInfo>();
            var script = new Helpers.ScriptHelper(kzApp.GetService<IScriptService>());

            using (var progress = new Progress((uint) lsList.Full.Count, "Получаем данные из ЛС"))
            {
                foreach (var lsElem in lsList.Full)
                {
                    if (progress.Increment())
                        break;
       
                    if(lsElem.Tn==524)
                        Thread.Sleep(1);
                    using (var ls = lsList.ReadLS(lsElem.Id, LSReadFlags.ReadOnly | LSReadFlags.GlobalLS))
                    {
                        var flCode = ls.Kch().GetValue(options.KchNumFromPseudo("FLCode")).Value.ToInt();
                        foreach (var ispDoldn in ls.IspDoljnList)
                        {
                            var combine = ispDoldn.Key;
                            var dolgnCodeList = ls.Kch(combine).GetValue(options.KchNumFromPseudo("DolgCode")).Value;
                            var dolgnCode = dolgnCodeList.Split(';').LastOrDefault()?.ToInt()??0;
                            foreach (var rowNu in ls.RowNUList.FiltrCombine(combine))
                            {
                                var bz = script.Evaluate<double>("double v;poiskbzm(code, mes, v, 1);return v;", "code", rowNu.IntCode, "mes", 0/*mtek*/);
                                if(rowNu.SumVect.Count>0)
                                    foreach (var sumElem in rowNu.SumVect)
                                    {
                                        if ( sumElem.mv != mtek || //месяц не текущий И
                                             Math.Abs(sumElem.Sum) < 0.001 && 
                                             Math.Abs(sumElem.rv)  < 0.001 && 
                                             rowNu.BaseValue<0.001) //сумма, РВ и БЗ нулевые
                                            continue;
                                        res.Add(new SummInfo
                                        {
                                            Tn = ls.Tn,
                                            DepartmentCode = lsElem.IdPodr,
                                            CombineId = combine,
                                            FLCode = flCode,
                                            DolgCode = dolgnCode,
                                            Fio=ls.Kch().GetFIO().TrimEnd(),
                                            IstCode =rowNu.Src,
                                            
                                            Snils = ls.Kch().GetValue(KchNum.SNILS).Value,
                                            Frv= Math.Round(sumElem.rv,2),
                                            Nrv = Math.Round(script.Call<double>("UserGetNrv", sumElem.m.AbsMonthToDate().ToInt()/ (double)100, combine,2),2),
                                            Bz= Math.Round(bz,2),
                                            Summ = sumElem.Sum,
                                            MNach = sumElem.mv,
                                            MPrin = sumElem.m,
                                            NuCode = (int)rowNu.IntCode,
                                            ObjCode=rowNu.Subject
                                        });
                                    }
                                else if(bz>0.001)
                                {
                                    res.Add(new SummInfo
                                    {
                                        Tn = ls.Tn,
                                        DepartmentCode = lsElem.IdPodr,
                                        CombineId = combine,
                                        FLCode = flCode,
                                        DolgCode = dolgnCode,
                                        Fio = ls.Kch().GetFIO().TrimEnd(),
                                        IstCode = rowNu.Src,

                                        Snils = ls.Kch().GetValue(KchNum.SNILS).Value,
                                        Frv = 0,
                                        Nrv = Math.Round(script.Call<double>("UserGetNrv", options.SystemMonth.AbsMonthToDate().ToInt() / (double)100, combine, 2), 2),
                                        Bz = Math.Round(bz, 2),
                                        Summ = 0,
                                        MNach = options.SystemMonth,
                                        MPrin = options.SystemMonth,
                                        NuCode = (int)rowNu.IntCode,
                                        ObjCode = rowNu.Subject
                                    });
                                }
                            }
                        }
                    }

                }
            }

            return res;
        }

        public static void FillRefs(IKZ kzApp, out Dictionary<int, string> dolgnNameDic,out Dictionary<int, int> tnDic)
        {
            dolgnNameDic = new Dictionary<int, string>();
            tnDic = new Dictionary<int, int>();
            var options = kzApp.GetService<IOptions>();
            var lsList = kzApp.GetService<ILsList>();
            var script = new ScriptHelper(kzApp.GetService<IScriptService>());
            foreach (var lsElem in lsList.Full)
            {
                using (var ls = lsList.ReadLS(lsElem.Id, LSReadFlags.ReadOnly | LSReadFlags.GlobalLS))
                {
                    foreach (var ispDoldn in ls.IspDoljnList)
                    {
                        var dolgnCodeList = ls.Kch(ispDoldn.Key).GetValue(options.KchNumFromPseudo("DolgCode")).Value;
                        foreach (var dolgnCode in dolgnCodeList.Split(';'))
                        {
                            var code = dolgnCode.ToInt();
                            if (!dolgnNameDic.ContainsKey(code))
                            {
                                var dolgnName = script.Evaluate<string>("char s[256];GetKchDoljn(s, 256, KDateFromInt_LastDay(mtek), combine);return OemToAnsi(s);", "combine", ispDoldn.Key).TrimEnd();
                                dolgnNameDic.Add(code, dolgnName);
                            }
                            if (!tnDic.ContainsKey(code))
                                tnDic.Add(code, ls.Tn);
                        }
                    }
                }
            }
        }

        public static void GroupData(ref List<SummInfo> data)
        {
            var groupedData = new List<SummInfo>();
            //группируем данные по ключу
            foreach (var pair in data.GroupBy(p=>p.Key))
            {
                var firstInGroup = pair.First();
                groupedData.Add(new SummInfo()
                {
                    DepartmentCode = firstInGroup.DepartmentCode,
                    FLCode = firstInGroup.FLCode,
                    DolgCode = firstInGroup.DolgCode,
                    Tn= firstInGroup.Tn,
                    Fio = firstInGroup.Fio,
                    Snils = firstInGroup.Snils,
                    IstCode = firstInGroup.IstCode,
                    ObjCode = firstInGroup.ObjCode,
                    MPrin = firstInGroup.MPrin,
                    MNach = firstInGroup.MNach,
                    NuCode = firstInGroup.NuCode,
                    NuName = firstInGroup.NuName,
                    Nrv = firstInGroup.Nrv,
                    CombineId = firstInGroup.CombineId,
                    Summ = pair.Sum(p => p.Summ),
                    Frv = pair.Sum(p => p.Frv),
                    Bz = pair.Sum(p => p.Bz),
                });
            }
            data = groupedData.ToList();
        }

        public static void CalcBz(List<SummInfo> data_1C)
        {
            foreach (var summInfo in data_1C)
                if (summInfo.IsNadbavka()  && data_1C.GetOklad(summInfo.CombineId, out double oklad))
                    summInfo.Bz = (100 * summInfo.Summ / oklad).Round(2); //вычисляем % надбавки
        }
        public static bool GetOklad(this List<SummInfo> data, int combineID, out double oklad)
        {
            oklad = 0;
            var item = data.Find(p => p.CombineId == combineID && p.NuCodeForCompare == 196);
            if (item == null)
                return false;
            oklad = item.Summ;
            return true;
        }

        public static bool IsNadbavka(this SummInfo data)
        {
            /*
             116 Доплата за расширение зоны обслуживания
             122 -_Доплата от оклада %
                 123, //_за непрерывный стаж
                 21,  //Надбавка за вредность
                 178, //Выслуга лет
                 124,//_Надбавка за категорию
                 127,//_Сложность напряженность
                 136,//Надбавка за квалификационную категорию
               */
            return data.NuCodeForCompare == 116 ||
                   data.NuCodeForCompare == 122 ||
                   data.NuCodeForCompare == 123 ||
                   data.NuCodeForCompare == 21  ||
                   data.NuCodeForCompare == 178 ||
                   data.NuCodeForCompare == 124 ||
                   data.NuCodeForCompare == 127 ||
                   data.NuCodeForCompare == 136;

        }
    }
}
