﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using SKBKontur.KZ.Common;
using SKBKontur.KZ.Core;
using SKBKontur.KZ.OfficeInterop;

namespace SKBKontur.KZ.User.CompareSumms.AllSummCompare
{
    public static class PrintHelper
    {
        public static void PrintDifferenceWithSrc(List<SummInfo> data_1C, List<SummInfo> data_kz, IOptions options,IReferences refs, Dictionary<int, string> dolgnNameDic, bool byDepapartment, bool hideSrcObjColumn)
        {
            var office = new OfficeService { PreferredOffice = options.PreferredOffice() };
            var dir = $"C:\\Raschet\\_Reports\\{options.SystemMonth.AbsMonthToDate():yyyy_MM}";
            if (!Directory.Exists(dir))
                Directory.CreateDirectory(dir);
            if (byDepapartment)
            {
                var printedCount = 0;
                var deps = data_1C.Select(p => p.DepartmentCode).Union(data_kz.Select(p => p.DepartmentCode));
                using (var progress = new Progress((uint)deps.Count(), "Печать подразделений"))
                {
                    foreach (var dep in deps)
                    {
                        if (progress.Update($"Печать подразделений ({++printedCount} из {deps.Count()})")|| progress.Increment())
                            break;
                        refs.Departments.TryGetValue(dep, out var depInfo);
                        var wbName =$"{depInfo?.ExtKod??dep.ToString()} {depInfo?.Name}".Replace('/',' ').Replace('\\', ' ').TrimEnd();
                        var wbPath = Path.Combine(dir, wbName);
                        Print_Impl(data_1C.Where(p => p.DepartmentCode == dep).ToList(),data_kz.Where(p => p.DepartmentCode == dep).ToList(), refs, dolgnNameDic,  office, wbPath, hideSrcObjColumn);
                    }
                }
            }
            else
            {
                var wbPath = Path.Combine(dir, "Все подразделения");
                Print_Impl(data_1C, data_kz, refs, dolgnNameDic, office, wbPath, hideSrcObjColumn);
            }
        }

        private static void Print_Impl(List<SummInfo> data_1C, List<SummInfo> data_kz, IReferences refs,Dictionary<int, string> dolgnNameDic, OfficeService office, string wbPath, bool hideSrcObjColumn)
        {
            using (var wb = office.CreateWorkbook()) //создали книгу
            {
                try
                {
                    Print_Impl(data_1C, data_kz, dolgnNameDic,  refs, wb.ActiveWorksheet,  hideSrcObjColumn);
                }
                finally
                {
                    wb.GetWorksheet(1).Activate();
                     //wb.Visible = true;
                    
                    wb.SaveAs($"{wbPath}.xls",OverwriteMode.Overwrite);
                    wb.Close();
                    wb.Detach();
                }
            }
        }

        private static void Print_Impl(List<SummInfo> data_1C, List<SummInfo> data_kz,Dictionary<int, string> dolgnNameDic, IReferences refs, IWorksheet wsh, bool hideSrcObjColumn)
        {
            var allKeys = data_1C.Select(p=>p.Key).Union(data_kz.Select(p=>p.Key)).ToArray();
            Array.Sort(allKeys);
            var row = 1;
            var col = 0;
            wsh.Cell(row, ++col).Value = "Ключ";   //1
            wsh.Cell(row, ++col).Value = "ТН"; //2
            wsh.Cell(row, ++col).Value = "ФИО";//3
            wsh.Cell(row, ++col).Value = "Код должности";//4
            wsh.Column(col).NumberFormat = "@";
            wsh.Cell(row, ++col).Value = "Название должности";//5
            wsh.Cell(row, ++col).Value = "Источник";//6
            wsh.Cell(row, ++col).Value = "Статья затрат";//7
            wsh.Cell(row, ++col).Value = "Код вида";//8
            wsh.Cell(row, ++col).Value = "Сумма 1С";//9
            wsh.Cell(row, ++col).Value = "Сумма КЗ";//10
            wsh.Cell(row, ++col).Value = "Разница";//11
            wsh.Cell(row, ++col).Value = "В пределах допустимой погрешности";//12
            wsh.Cell(row, ++col).Value = "Базовое значение 1C";//13
            wsh.Cell(row, ++col).Value = "Базовое значение КЗ";//14
            wsh.Cell(row, ++col).Value = "ФРВ КЗ";//15
            wsh.Cell(row, ++col).Value = "НРВ КЗ";//16
            using (var progress = new Progress((uint) allKeys.Length, "Печать различий в Excel"))
            {
                foreach (var key in allKeys)
                {
                    if (progress.Increment())
                        break;
                    row++;
                    col = 0;
                    wsh.Cell(row, ++col).Value = key;
                    var data1C = data_1C.SingleOrDefault(p =>string.Compare(p.Key,key,StringComparison.InvariantCultureIgnoreCase)==0);
                    var dataKz = data_kz.SingleOrDefault(p => string.Compare(p.Key, key, StringComparison.InvariantCultureIgnoreCase) == 0);
                    var dolgCode = data1C?.DolgCode ?? dataKz.DolgCode;
                    wsh.Cell(row, ++col).Value = data1C?.Tn ?? dataKz.Tn;
                    wsh.Cell(row, ++col).Value = data1C?.Fio ?? dataKz.Fio;
                    if (!dolgnNameDic.TryGetValue(dolgCode, out var dolgName))
                        dolgName = dolgCode.ToString();
                    var combine = data1C?.CombineId ?? dataKz.CombineId;
                    wsh.Cell(row, ++col).Value =$"{combine}:{dolgCode}";
                    wsh.Cell(row, ++col).Value = dolgName;
                    var nuCode = data1C?.NuCodeForCompare ?? dataKz.NuCodeForCompare;
                    //источник
                    var istCode = data1C?.IstCode ?? dataKz.IstCode;
                    refs.Sources.TryGetValue(istCode, out var src);
                    wsh.Cell(row, ++col).Value = $"{src?.ExtKod} {src?.Name}";
                    //объект
                    var objCode = data1C?.ObjCode ?? dataKz.ObjCode;
                    refs.Subjects.TryGetValue(objCode, out var obj);
                    wsh.Cell(row, ++col).Value = $"{obj?.ExtKod} {obj?.Name}";

                    refs.Tvx.TryGetValue((ushort)nuCode,out var nuInfo);
                    wsh.Cell(row, ++col).Value = $"{nuInfo?.ExtCode} {nuInfo?.LongName}";
                    wsh.Cell(row, ++col).Value = data1C?.Summ ?? 0;
                    wsh.Cell(row, ++col).Value = dataKz?.Summ ?? 0;
                    wsh.Cell(row, ++col).Value = Math.Abs((data1C?.Summ ?? 0) - (dataKz?.Summ ?? 0));
                    var v = Math.Abs((data1C?.Summ ?? 0) - (dataKz?.Summ ?? 0)) <0.01 ? 1 : 0;
                    wsh.Cell(row, ++col).Value = v;    //допустимая погрешность 0 коп.

                    if (data1C != null)
                        wsh.Cell(row, 13).Value = data1C.Bz;
                    if (dataKz != null)
                    {
                        wsh.Cell(row, 14).Value = dataKz.Bz;
                        wsh.Cell(row, 15).Value = dataKz.Frv; 
                        wsh.Cell(row, 16).Value = dataKz.Nrv;
                    }
                }
            }
            for (var i = 2; i <= 16; i++)
            {
                wsh.Column(i).AutoFitWidth();
            }
            wsh.Row(1).AutoFilter();

            if (hideSrcObjColumn)
            {
                wsh.Column(6).ColumnWidth = 0;
                wsh.Column(7).ColumnWidth = 0;
            }
        }
    }
}
