﻿using System.Collections.Generic;
using System.Linq;
using SKBKontur.KZ.Common;
using SKBKontur.KZ.Core;
using SKBKontur.KZ.User.LsData;

namespace SKBKontur.KZ.User.CompareSumms.AvansCompare
{
    class DataHelper
    {
        public static List<SummAvansInfo> GetKzData(int kzNuCode, IKZ kzApp)
        {
            var options = kzApp.GetService<IOptions>();
            var lsList = kzApp.GetService<ILsList>();
            var mtek = options.SystemMonth;

            var res=new List<SummAvansInfo>();
            foreach (var lsElem in lsList.Full)
            {
                using (var ls = lsList.ReadLS(lsElem.Id, LSReadFlags.ReadOnly | LSReadFlags.GlobalLS))
                {

                    var rows = ls.RowNUList.Where(p => p.Code.Vid == kzNuCode);
                    double summ = 0;
                    foreach (var row in rows)
                    {
                        foreach (var sumElem in row.SumVect)
                        {
                            if (sumElem.mv != mtek)
                                continue;
                            summ += sumElem.Sum;
                        }
                    }
                    if (summ < 0.001)
                        continue;
                    res.Add(new SummAvansInfo()
                    {
                        FLCode = ls.Kch().GetValue(options.KchNumFromPseudo("FLCode")).Value.ToInt(),
                        Fio = ls.Kch().GetFIO().TrimEnd(),
                        Snils = ls.Kch().GetValue(KchNum.SNILS).Value,
                        Summ = summ
                    });
                }
            }
            return res;
        }
    }
    public static class CastHelper
    {
        public static List<SummAvansInfo> CastToSummAvansInfo(this List<SummInfo> list)
        {
            var res=new List<SummAvansInfo>();
            foreach (var elems in list.GroupBy(p=>p.FLCode))
            {
                var summ = elems.Sum(elem => elem.Summ);
                res.Add(new SummAvansInfo()
                {
                    Fio = elems.First().Fio.TrimEnd(),
                    Snils = elems.First().Snils,
                    FLCode = elems.Key,
                    Summ = summ
                });
            }
            return res;
        }
    }
}
