﻿using System;
using System.Collections.Generic;
using System.Linq;
using SKBKontur.KZ.Common;
using SKBKontur.KZ.Core;
using SKBKontur.KZ.OfficeInterop;

namespace SKBKontur.KZ.User.CompareSumms.AvansCompare
{
    class PrintHelper
    {
        public static void PrintDifference(IEnumerable<SummAvansInfo> data_1C, List<SummAvansInfo> dataKz, IOptions options)
        {
            var office = new OfficeService {PreferredOffice = options.PreferredOffice()};
            using (var wb = office.CreateWorkbook()) //создали книгу
            {
                try
                {
                    Print_Impl(data_1C,dataKz,wb.ActiveWorksheet);
                }
                finally
                {
                    wb.GetWorksheet(1).Activate();
                    wb.Visible = true;
                    wb.Detach();
                }
            }
        }



        private static void Print_Impl(IEnumerable<SummAvansInfo> data1C, List<SummAvansInfo> dataKz, IWorksheet wsh)
        {
            var summAvansInfos = data1C as SummAvansInfo[] ?? data1C.ToArray();
            var allKeys = summAvansInfos.Select(p => p.Key).Union(dataKz.Select(p => p.Key)).ToArray();
            Array.Sort(allKeys);
            var row = 1;
            wsh.Cell(row, "A").Value = "Ключ";
            wsh.Cell(row, "B").Value = "СНИЛС";
            wsh.Cell(row, "C").Value = "ФИО";
            wsh.Cell(row, "D").Value = "Сумма 1С";
            wsh.Cell(row, "E").Value = "Сумма КЗ";
            wsh.Cell(row, "F").Value = "Разница";
            using (var progress =new Progress((uint)allKeys.Length,"Печать различий в Excel"))
            {
                foreach (var key in allKeys)
                {
                    progress.Increment();
                    row++;
                    wsh.Cell(row, "A").Value = key;
                    var data1 = summAvansInfos.SingleOrDefault(p => p.Key.Contains(key));
                    var data2 = dataKz.SingleOrDefault(p => p.Key.Contains(key));
                    wsh.Cell(row, "B").Value = data1?.Snils ?? data2.Snils;
                    wsh.Cell(row, "C").Value = data1?.Fio ?? data2.Fio;
                    wsh.Cell(row, "D").Value = data1?.Summ ?? 0;
                    wsh.Cell(row, "E").Value = data2?.Summ ?? 0;
                    wsh.Cell(row, "F").Value = (data1?.Summ ?? 0) - (data2?.Summ ?? 0);
                }
            }
            var itog1 = data1C.Sum(p => p.Summ);
            var itog2 = dataKz.Sum(p => p.Summ);
            row++;
            wsh.Cell(row, "D").Value = itog1;
            wsh.Cell(row, "E").Value = itog2;
            wsh.Cell(row, "F").Value = itog1 - itog2;

        }
    }
}
