﻿using System.Collections.Generic;
using System.Linq;
using SKBKontur.KZ.Common;
using SKBKontur.KZ.Core;
using SKBKontur.KZ.User.Helpers;
using SKBKontur.KZ.User.LsData;
namespace SKBKontur.KZ.User.CompareSumms.PlanNuCompare
{
    public static class DataHelper
    {

        public static List<PlanSummInfo> GetKzPlanSumm(IKZ kzApp)
        {
            var options = kzApp.GetService<IOptions>();
            var lsList = kzApp.GetService<ILsList>();
            var mtek = options.SystemMonth;
            var lastDayInMonth = mtek.AbsMonthToDate().ToLastDayInMonth();
            var script = new ScriptHelper(kzApp.GetService<IScriptService>());
            var res = new List<PlanSummInfo>();

            using (var progress = new Progress((uint)lsList.Full.Count, "Получаем данные из ЛС"))
            {
                foreach (var lsElem in lsList.Full)
                {
                    if(progress.Increment())
                        break;
                    using (var ls = lsList.ReadLS(lsElem.Id, LSReadFlags.ReadOnly | LSReadFlags.GlobalLS))
                    {
                        var flCode = ls.Kch().GetValue(options.KchNumFromPseudo("FLCode")).Value.ToInt();
                        foreach (var ispDoldn in ls.IspDoljnList)
                        {
                            var combine = ispDoldn.Key;
                            var dolgnCode = ls.Kch(combine).GetValue(options.KchNumFromPseudo("DolgCode")).Value.Split(';').LastOrDefault()?.ToInt()??0;
                            foreach (var rowNU in ls.RowNUList.ValidOn(lastDayInMonth).FiltrCombine(combine).Where(row => row.BaseValue > double.Epsilon))
                            {
                                var summ = script.Evaluate<double>("double v;poiskbzm(code, mes, v, 1);return v;", "code", rowNU.IntCode, "mes", mtek);
                                res.Add(new PlanSummInfo
                                    {
                                        FLCode = flCode,
                                        Fio = ls.Kch().GetFIO(),
                                        DolgCode = dolgnCode,
                                        NuCode = (int)rowNU.IntCode,
                                        Summ = summ,
                                        Bz = rowNU.BaseValue
                                    }
                                );
                            }
                        }
                    }
                }
            }

            
            return res;
        }

        public static void CalcBz(List<PlanSummInfo> data_1C)
        {
            foreach (var planSummInfo in data_1C)
            {
                if (planSummInfo.NuCode == 196)
                {
                    planSummInfo.Bz = planSummInfo.Summ;
                    continue;
                }
                if(data_1C.GetOklad(planSummInfo.FLCode, out double oklad))
                   planSummInfo.Bz = (100*planSummInfo.Summ/oklad).Round(2); //вычисляем % надбавки
                else
                    planSummInfo.Bz = planSummInfo.Summ; //чтобы ничего не потерять - пишем в БЗ сумму
            }
        }
        public static bool GetOklad(this List<PlanSummInfo> data, int flCode, out double oklad)
        {
            oklad = 0;
            var item = data.Find(p => p.FLCode == flCode && p.NuCode == 196);
            if (item == null)
                return false;
            oklad= item.Summ;
            return true;
        }

        public static void FillRefs(IKZ kzApp, out Dictionary<int, string> dolgnNameDic, out Dictionary<int, int> tnDic,out Dictionary<int, double> stavkaDic)
        {
            dolgnNameDic = new Dictionary<int,string>();
            tnDic = new Dictionary<int, int>();
            stavkaDic=new Dictionary<int, double>();
            var options = kzApp.GetService<IOptions>();
            var lsList = kzApp.GetService<ILsList>();
            
            var script = new ScriptHelper(kzApp.GetService<IScriptService>());
            foreach (var lsElem in lsList.Full)
            {
                using (var ls = lsList.ReadLS(lsElem.Id, LSReadFlags.ReadOnly | LSReadFlags.GlobalLS))
                {
                    foreach (var ispDoldn in ls.IspDoljnList)
                    {
                        var dolgnCodeList = ls.Kch(ispDoldn.Key).GetValue(options.KchNumFromPseudo("DolgCode")).Value;
                        foreach (var dolgnCode in dolgnCodeList.Split(';'))
                        {
                            var code = dolgnCode.ToInt();
                            if (!dolgnNameDic.ContainsKey(code))
                            {
                                var dolgnName = script.Evaluate<string>("char s[256];GetKchDoljn(s, 256, KDateFromInt_LastDay(mtek), combine);return OemToAnsi(s);", "combine", ispDoldn.Key).TrimEnd();
                                dolgnNameDic.Add(code, dolgnName);
                            }
                            if (!tnDic.ContainsKey(code))
                                tnDic.Add(code, ls.Tn);
                            if (!stavkaDic.ContainsKey(code))
                            {
                                var stavka = script.Evaluate<double>("return DefineKEmploy(combine,KDateFromInt_LastDay(mtek));", "combine", ispDoldn.Key);
                                stavkaDic.Add(code, stavka);
                            }
                        }
                    }
                }
            }
        }

        public static void GroupData(ref List<PlanSummInfo> data_1C)
        {
            var groupedData = new List<PlanSummInfo>();
            //группируем данные по ключу
            foreach (var pair in data_1C.GroupBy(p => p.Key()))
            {
                var firstInGroup = pair.First();
                groupedData.Add(new PlanSummInfo()
                {
                    FLCode = firstInGroup.FLCode,
                    DolgCode = firstInGroup.DolgCode,
                    DolgName = firstInGroup.DolgName,
                    NuCode = firstInGroup.NuForCompare(),
                    NuName = firstInGroup.NuName,
                    Fio = firstInGroup.Fio,
                    Snils = firstInGroup.Snils,
                    Bz = pair.Sum(p => p.Bz),
                    Summ = pair.Sum(p => p.Summ)
                });
            }
            data_1C = groupedData.ToList();
        }

        public static List<PlanSummInfo> GetPlanSumm(string fileName, IKZ kzApp)
        {
            return new ExcelDataGetter(kzApp).LoadData(fileName,out List<PlanSummInfo> res) ? res : null;
        }


    }
}
