﻿using System;
using System.Collections.Generic;
using SKBKontur.KZ.Common;
using SKBKontur.KZ.Core;
using SKBKontur.KZ.OfficeInterop;
using SKBKontur.KZ.User.Helpers;
using SKBKontur.KZ.User.LsData;

namespace SKBKontur.KZ.User.CompareSumms.PlanNuCompare
{
    class ExcelDataGetter
    {
        private readonly IKZ _kzApp;

        public ExcelDataGetter(IKZ kzApp)
        {
            _kzApp = kzApp;
        }

        public bool LoadData(string filePath, out List<PlanSummInfo> data)
        {
            data = new List<PlanSummInfo>();
            var office = new OfficeService {PreferredOffice = _kzApp.GetService<IOptions>().PreferredOffice()};
            using (var wb = office.OpenWorkbook(filePath))
            {
                var wsh = wb.ActiveWorksheet;
                try
                {
                    LoadDataImpl(wsh, data);
                }
                catch (Exception ex)
                {
                    return false;
                }
                finally
                {
                    wb.Visible = false;
                    wb.Close();
                    wb.Detach();
                }

                return true;
            }
        }

        private void LoadDataImpl(IWorksheet wsh, List<PlanSummInfo> data)
        {
            var row = 14;
            var rowCount =  wsh.UsedRange().RowsCount;
            PlanSummInfo curData = null;
            var script = new ScriptHelper(_kzApp.GetService<IScriptService>());
            using (var progress = new Progress((uint) rowCount, "Обрабатываем данные Excel..."))
            {
                while (row < rowCount)
                {
                    if (FindDolgnRow(wsh, row)) //строка с фио и названием должности
                    {
                        curData = new PlanSummInfo()
                        {
                            FLCode = DefineFlCode(wsh, row),
                            DolgCode = DefineDolgCode(wsh, row),
                            DolgName = DefineDolgName(wsh, row),
                            Fio = DefinFio(wsh, row)
                        };
                        
                    }

                    if (FindNuRow(wsh, row))
                    {
                        curData = (PlanSummInfo)curData.Clone();
                        curData.NuName = DefineNuName(wsh, row);
                        curData.NuCode = script.Call<int>("DefineNuCode", curData.NuName); 
                        curData.Summ = DefineNuSumm(wsh, row);
                        data.Add(curData);
                    }
                    row++;
                    if (progress.Update((uint)row))
                        break;

                }


            }
        }
        private double DefineNuSumm(IWorksheet wsh, int row)
        {
            return wsh.Cell(row, 17).Value.ToString().Replace(',','.').ToDouble();
        }

        private string DefineNuName(IWorksheet wsh, int row)
        {
          return wsh.Cell(row, 1).Value.ToString().TrimEnd();
        }

        private int DefineNuCode(IWorksheet wsh, int row)
        {
            throw new NotImplementedException();
        }

        private bool FindNuRow(IWorksheet wsh, int row)
        {
            var nuName = wsh.Cell(row, 1).Value?.ToString().TrimEnd();
            var dolgName = DefineDolgName(wsh,row);
            var summ = wsh.Cell(row, 17).Value?.ToString().ToDouble();
            return nuName != null && string.IsNullOrWhiteSpace(dolgName) && nuName.Length > 1 && summ != null && summ > 0.001;

        }

        private string DefineDolgName(IWorksheet wsh, int row)
        {
            var value = wsh.Cell(row, 16).Value?.ToString();
            return value;
        }

        private string DefinFio(IWorksheet wsh, int row)
        {
            var value = wsh.Cell(row, "A").Value.ToString().Split(',')[0];
            return value;
        }

        private int DefineDolgCode(IWorksheet wsh, int row)
        {
            var value = wsh.Cell(row, "A").Value.ToString().Split(',')[2];
            return value.MyToInt();
        }

        private int DefineFlCode(IWorksheet wsh, int row)
        {
            var value = wsh.Cell(row, "A").Value.ToString().Split(',')[1];
            return value.MyToInt();

        }

        private bool FindDolgnRow(IWorksheet wsh, int row)
        {
            var values = wsh.Cell(row, "A").Value.ToString().Split(',');
            return values.Length == 3;//в Строке через разделитель "запятая" указаны "фио","код физ лица"," код должности":Петров Иван Иванович, 00-0001131, 0000001533
        }
    }
}