﻿
using System;
using System.Collections.Generic;
using System.Linq;
using SKBKontur.KZ.Common;
using SKBKontur.KZ.Core;
using SKBKontur.KZ.OfficeInterop;
using SKBKontur.KZ.User.LsData;

namespace SKBKontur.KZ.User.CompareSumms.PlanNuCompare
{
    public class PrintHelper
    {
        internal static void PrintDifference(List<PlanSummInfo> data_1c, List<PlanSummInfo> data_kz, IOptions options,Dictionary<int, string> dolgnNameDic, Dictionary<int, int> tnDic, Dictionary<int, double> stavkaDic)
        {
            var office = new OfficeService { PreferredOffice = options.PreferredOffice() };
            using (var wb = office.CreateWorkbook()) //создали книгу
            {
                try
                {
                    Print_Impl(data_1c, data_kz, dolgnNameDic, tnDic, stavkaDic, wb.ActiveWorksheet);
                }
                finally
                {
                    wb.GetWorksheet(1).Activate();
                    wb.Visible = true;
                    wb.Detach();
                }
            }
        }

        private static void Print_Impl(List<PlanSummInfo> data_1C, List<PlanSummInfo> data_Kz,Dictionary<int, string> dolgnNameDic, Dictionary<int, int> tnDic, Dictionary<int, double> stavkaDic,IWorksheet wsh)
        {
            var allKeys = data_1C.Select(p => p.Key()).Union(data_Kz.Select(p => p.Key())).ToArray();
            Array.Sort(allKeys);
            var row = 1;
            wsh.Cell(row, "A").Value = "Ключ";
            wsh.Cell(row, "B").Value = "ТН";
            wsh.Cell(row, "C").Value = "ФИО";
            wsh.Cell(row, "D").Value = "Название должности";
            wsh.Cell(row, "E").Value = "Код вида";
            wsh.Cell(row, "F").Value = "Сумма 1С";
            wsh.Cell(row, "G").Value = "Сумма КЗ";
            wsh.Cell(row, "H").Value = "Разница";
            wsh.Cell(row, "I").Value = "Оклад с учетом ставки";
            wsh.Cell(row, "J").Value = "В пределах допустимой погрешности";
            using (var progress = new Progress((uint)allKeys.Length, "Печать различий в Excel"))
            {
                foreach (var key in allKeys)
                {
                    if(progress.Increment())
                        break;
                    row++;
                    wsh.Cell(row, "A").Value = key;
                    var data1C = data_1C.SingleOrDefault(p => p.Key().Contains(key));
                    var dataKz = data_Kz.SingleOrDefault(p => p.Key().Contains(key));
                    var dolgCode = data1C?.DolgCode ?? dataKz.DolgCode;
                    tnDic.TryGetValue(dolgCode, out int tn);
                    wsh.Cell(row, "B").Value = tn;
                    wsh.Cell(row, "C").Value = data1C?.Fio ?? dataKz.Fio;

                    if (!dolgnNameDic.TryGetValue(dolgCode, out var dolgName))
                        dolgName = dolgCode.ToString();
                    wsh.Cell(row, "D").Value = dolgName;
                    var nuCode = data1C?.NuCode ?? dataKz.NuForCompare();
                    wsh.Cell(row, "E").Value = data1C?.NuCode ?? dataKz.NuForCompare();
                    wsh.Cell(row, "F").Value = data1C?.Bz ?? 0;
                    wsh.Cell(row, "G").Value = dataKz?.Summ ?? 0;
                    wsh.Cell(row, "H").Value = (data1C?.Bz ?? 0) - (dataKz?.Summ ?? 0);
                    var delta = nuCode == 196 ? 1000 : 1.01;
                    var v = Math.Abs((data1C?.Bz ?? 0) - (dataKz?.Summ ?? 0)) > delta ? 0 : 1;

                    if (!stavkaDic.TryGetValue(dolgCode, out var stavka))
                        stavka = 1;
                    if (stavka < 0.001)
                        stavka = 1;
                    wsh.Cell(row, "I").Value = dataKz!=null && dataKz.IsOklad()? dataKz.Summ / stavka:0;   
                    wsh.Cell(row, "J").Value =v;    //для окладов допустимая погрешность 1000. для надбавок - погрешность 1%
                }
            }
            for (var i = 2; i <= 9; i++)
            {
                wsh.Column(i).AutoFitWidth();
            }
            wsh.Row(1).AutoFilter();
        }
    }
}
