﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using SKBKontur.KZ.COM.Bank.Common.Helpers;
using SKBKontur.KZ.Common;
using SKBKontur.KZ.Core;
using SKBKontur.KZ.Forms;
using SKBKontur.KZ.User.LsData;
using usefulLib;

namespace SKBKontur.KZ.User.Helpers
{
   public static partial class LsDataHelper
   {
      public static List<LsInfo> GetLsData(DataTable table, TnHelper tnHelper)
      {
         var res = new List<LsInfo>();
         if (!table.Columns.Contains("FLCode") ||
             !table.Columns.Contains("PodrCode") ||
             !table.Columns.Contains("DolgCode") ||
             !table.Columns.Contains("DOLGGUID") ||
             !table.Columns.Contains("F") ||
             !table.Columns.Contains("I") ||
             !table.Columns.Contains("O") ||
             !table.Columns.Contains("SNILS") ||
             !table.Columns.Contains("VidDolgn") ||
             !table.Columns.Contains("DPriem") ||
             !table.Columns.Contains("DUvoln") ||
             !table.Columns.Contains("DolgName") ||
             !table.Columns.Contains("Stavka") ||
             !table.Columns.Contains("GrName"))
            throw new Exception("Не удалось найти данные о исп.должностях");

         foreach (DataRow dataRow in table.Rows)
         {
            var flCode = dataRow["FLCode"].ToString().TrimEnd();
            var podrCode = dataRow["PodrCode"].ToString().MyToInt();
            var dolgCode = dataRow["DolgCode"].ToString().TrimEnd();
            var dolgnGuid = dataRow["DOLGGUID"].ToString().TrimEnd();
            var fam = dataRow["F"].ToString().TrimEnd();
            var im = dataRow["I"].ToString().TrimEnd();
            var otch = dataRow["O"].ToString().TrimEnd();
            var snils = dataRow["SNILS"].ToString().TrimEnd();
            var vidDolgn = dataRow["VidDolgn"].ToString().TrimEnd();
            var dPriem = dataRow["DPriem"].ToString().ToKDate().IsValid()
                ? dataRow["DPriem"].ToString()
                : Consts.EmptyDateString;
            var dUvoln = dataRow["DUvoln"].ToString().ToKDate().IsValid()
                ? dataRow["DUvoln"].ToString()
                : Consts.EmptyDateString;
            var dolgName = dataRow["DolgName"].ToString().TrimEnd();
            var stavka = (double)dataRow["Stavka"];
            var grName = dataRow["GrName"].ToString().TrimEnd();
            res.Add(new LsInfo
            {
               Tn = -1,
               PodrCode = podrCode,
               F = fam,
               I = im,
               O = otch,
               Snils = snils,
               DolgCode = dolgCode,
               DolgGuid = dolgnGuid,
               DolgName = dolgName,
               VidDolgn = vidDolgn,
               DPriem = dPriem,
               DUvoln = dUvoln,
               FLCode = flCode,
               Stavka = stavka,
               GraffikName = grName
            });
         }

         //сортируем
         res = res.OrderBy(GetOrderKey).ToList();
         //вычисляем ТН
         FillTn(res, tnHelper);

         return res;
      }

      /// <summary>
      /// Если исп. должность одна - возвращаем код этой исп. должности
      /// Иначе возвращаем код той исп.должности, у которойVidDolgn==Основное место работы
      /// </summary>
      /// <param name="data"></param>
      /// <param name="tnHelper"></param>
      private static void FillTn(List<LsInfo> data, TnHelper tnHelper)
      {
         //берем данные по уже созданным ЛС..
         tnHelper.FillFromLsList();

         //не можем взять код должности в качестве ТН..
         var badDolgnCode = new List<string>();

         foreach (var lsInfo in data)
         {
            //в DolgCode строка - обработаем такие данные на сл. эпапе
            if(!lsInfo.DolgCode.IsDigitsOnly())
            {
               badDolgnCode.Add(lsInfo.DolgCode);
               continue;
            }

            var tn=tnHelper.CalcTn(lsInfo.FLCode, lsInfo.DolgCode, data);
            lsInfo.Tn = tn;
         }
         //пробежим по оставшимся должностям и присвоим им новые ТН
         foreach (var lsInfo in data.Where(p => badDolgnCode.Contains(p.DolgCode) && p.Tn<1))
         {
            var tn = tnHelper.CalcTn(lsInfo.FLCode, lsInfo.DolgCode, data);
            lsInfo.Tn = tn;
         }
      }

      private static string GetOrderKey(LsInfo lsInfo)
      {
         //1)по коду физ лица
         //2)по дате увольнения
         //3)сначала основное место работы
         //4)внутренние сов-ва последние
         return
             $"{lsInfo.FLCode}_{lsInfo.DUvoln.ToKDate().ToInt()}_{(lsInfo.IsMain ? "0" : "1")}_{(lsInfo.IsSovm ? "1" : "0")}";
      }
      public static bool LsExist(ILsList lsList, int tn)
      {
         return lsList.Full.Any(ls => ls.Tn == tn);
      }

      public static bool CreateLs(IKZ kzApp, int tn, int departmentCode)
      {
         if (tn < 1)
            throw new ArgumentException($@"ТН сотрудника должен быть больше нуля: {tn} ",
                nameof(tn));
         if (departmentCode < 1)
            throw new ArgumentException(
                $@"Код подразделения должен быть больше нуля: {departmentCode} ",
                nameof(departmentCode));

         if (LsExist(kzApp.GetService<ILsList>(), tn))
            return false;
         if (!new DepartmentList(kzApp.GetService<IReferences>()).Exists(departmentCode))
            return false;
         using (var ls = kzApp.GetService<ILsList>().CreateNewLS())
         {
            using (var o = kzApp.GetService<IOptions>().SetInMem("DnAskLSAcs", "2", "warnings")
            ) //чтобы созданный ЛС видели все расчетчики
            {
               ls.Kch().SetValue(KchNum.Tn, new KchValue(tn.ToString()), Consts.EmptyDate);
               ls.Kch().SetValue(KchNum.Podr, new KchValue(departmentCode.ToString()), Consts.EmptyDate);
               ls.Save();
            }
         }

         return true;
      }

      public static List<KchInfo> GetKchData(DataTable table, IScriptService scriptService)
      {
         if (!table.Columns.Contains("FLCode") ||
             !table.Columns.Contains("FIO") ||
             !table.Columns.Contains("INN") ||
             !table.Columns.Contains("SNILS") ||
             !table.Columns.Contains("DataRogd") ||
             !table.Columns.Contains("MestRogd") ||
             !table.Columns.Contains("Pol") ||
             !table.Columns.Contains("AdresR") ||
             !table.Columns.Contains("AdresF") ||
             !table.Columns.Contains("PhoneM") ||
             !table.Columns.Contains("PhoneH") ||
             !table.Columns.Contains("PhoneW") ||
             !table.Columns.Contains("DocCode") ||
             !table.Columns.Contains("DocName") ||
             !table.Columns.Contains("DocSeria") ||
             !table.Columns.Contains("DocNumber") ||
             !table.Columns.Contains("DocDate") ||
             !table.Columns.Contains("DocKem") ||
             !table.Columns.Contains("DocPodr") ||
             !table.Columns.Contains("ZPPROJ") ||
             !table.Columns.Contains("CARDNUM") ||
             !table.Columns.Contains("Stag"))
            throw new Exception("Не удалось найти кадровые данные");

         var res = new List<KchInfo>();
         foreach (DataRow dataRow in table.Rows)
         {

            var flCode = dataRow["FLCode"].ToString().TrimEnd();
            var fio = dataRow["FIO"].ToString().TrimEnd();
            var inn = dataRow["INN"].ToString().TrimEnd();
            var snils = dataRow["SNILS"].ToString().TrimEnd();
            var dataRogd = dataRow["DataRogd"].ToString().TrimEnd();
            var mestoRogd = dataRow["MestRogd"].ToString().TrimEnd();
            var pol = dataRow["Pol"].ToString().Length < 1
                ? ""
                : dataRow["Pol"].ToString().Substring(0, 1); //первый символ
            var phoneM = dataRow["PhoneM"].ToString().TrimEnd();
            var phoneW = dataRow["PhoneW"].ToString().TrimEnd();
            var phoneH = dataRow["PhoneH"].ToString().TrimEnd();
            var docCode = dataRow["DocCode"].ToString().ToInt();
            var docName = dataRow["DocName"].ToString().TrimEnd();
            var docSeria = dataRow["DocSeria"].ToString().TrimEnd();
            var docNumber = dataRow["DocNumber"].ToString().TrimEnd();
            var docDate = dataRow["DocDate"].ToString().TrimEnd();
            var docKem = dataRow["DocKem"].ToString().TrimEnd();
            var docPodrCode = dataRow["DocPodr"].ToString().TrimEnd();
            var stagBl = "";
            var stagData = dataRow["Stag"].ToString().TrimEnd().Split(';');
            if (stagData.Length >= 3)
            {
               var date = stagData[0].ToKDate();
               if (date.IsValid())
               {
                  var totalMonth = stagData[1].ToInt();
                  var years = totalMonth / 12;
                  var months = totalMonth % 12;
                  var days = stagData[2].ToInt();
                  stagBl = $"{date.ToKDateString()};{years} {months} {days}";
               }
            }

            var adres = scriptService.CreateScriptObject<IAdres>();
            adres.CodePage = TCodePage.WinCodePage;
            var adresF = dataRow["AdresF"].ToString().TrimEnd();
            if (adresF.Length > 0)
               adresF = GetAdres(adres, adresF);
            var adresR = dataRow["AdresR"].ToString().TrimEnd();
            if (adresR.Length > 0)
               adresR = GetAdres(adres, adresR);



            var pasp = scriptService.CreateScriptObject<IPasport>();
            pasp.CodePage = TCodePage.WinCodePage;
            pasp.SetCodDoc(docCode);
            pasp.SetNumber(docNumber);
            pasp.SetSeries(docSeria);
            pasp.SetWhen(docDate);
            pasp.SetWho(docKem + " " + docPodrCode);
            var paspAsString = pasp.GetPasport("Д_НАЗВ");


            res.Add(new KchInfo()
            {
               FLCode = flCode,
               Fio = fio,
               Inn = inn,
               Snils = snils,
               DataRogd = dataRogd,
               Pol = pol,
               AdresFact = adresF,
               AdresReg = adresR,
               PhoneHome = phoneH,
               PhoneMobile = phoneM,
               PhoneWork = phoneW,
               DocCode = docCode,
               DocName = docName,
               DocSeria = docSeria,
               DocNumber = docNumber,
               DocDate = docDate,
               DocKem = docKem,
               DocPodrCode = docPodrCode,
               Pasport = paspAsString,
               MestoRogd = mestoRogd,
               StagBl = stagBl,
               BankName = dataRow["ZPPROJ"].ToString().TrimEnd(),
               BankCardNumber = dataRow["CARDNUM"].ToString().TrimEnd()
            });
         }

         return res;
      }

      private static string GetAdres(IAdres adres, string value)
      {
         var words = value.Split(',');
         if (words.Length < 5)
            return "";

         //хак
         if (string.Compare("РОССИЯ", words[0], StringComparison.InvariantCultureIgnoreCase) == 0)
            value = value.Substring(value.IndexOf(",") + 1);
         adres.Initial(value);

         var v = adres.GetAdres("ПОЛН_АДРЕС", "ПРОСТО");
         var v2 = adres.GetAdres("П_АДРЕС", "ПРОСТО");

         if (adres.GetAdres("ИНДЕКС", "ПРОСТО").ToInt() == 0)
            adres.SetIndex(0);
         return adres.GetAdres("ПОЛН_АДРЕС", "ПРОСТО");
      }

      public static List<TabelInfo> GetTabelData(DataTable table)
      {
         if (!table.Columns.Contains("FLCode") ||
             !table.Columns.Contains("DolgCode") ||
             !table.Columns.Contains("DOLGGUID") ||
             !table.Columns.Contains("Fio") ||
             !table.Columns.Contains("SNILS") ||
             !table.Columns.Contains("Date") ||
             !table.Columns.Contains("Sim") ||
             !table.Columns.Contains("Hours") ||
             !table.Columns.Contains("FullName"))
            throw new Exception("Не удалось найти данные о табеле");

         var res = new List<TabelInfo>();
         foreach (DataRow dataRow in table.Rows)
         {
            var dolgCode = dataRow["DolgCode"].ToString().TrimEnd();
            var dolgGuid = dataRow["DOLGGUID"].ToString().TrimEnd();
            var flCode = dataRow["FLCode"].ToString().TrimEnd();
            var fio = dataRow["Fio"].ToString().TrimEnd();
            var date = dataRow["Date"].ToString().TrimEnd();
            if (!date.ToKDate().IsValid())
            {
               MsgBox.ShowError($"Для {fio} неверно передана дата: {date}");
               continue;
            }

            var snils = dataRow["SNILS"].ToString().TrimEnd();
            var sim = dataRow["Sim"].ToString().TrimEnd();
            var hours = (double)dataRow["Hours"];
            var fullName = dataRow["FullName"].ToString().TrimEnd();

            res.Add(new TabelInfo()
            {
               FLCode = flCode,
               DolgCode = dolgCode,
               DolgGuid = dolgGuid,
               Fio = fio,
               Snils = snils,
               Sim = sim,
               Date = date,
               Hours = hours,
               FullName = fullName
            });
         }

         return res;
      }

      public static List<SummInfo> GetSummData(DataTable table, IScriptService scriptService)
      {
         try
         {
            if (!table.Columns.Contains("FLCode") ||
      !table.Columns.Contains("DolgCode") ||
      !table.Columns.Contains("Fio") ||
      !table.Columns.Contains("SNILS") ||
      !table.Columns.Contains("ISTNAME") ||
      !table.Columns.Contains("SUMM") ||
      !table.Columns.Contains("MPRIN") ||
      !table.Columns.Contains("MNACH") ||
      !table.Columns.Contains("NUNAME"))
               throw new Exception("Не удалось найти данные о суммах");

            var script = new ScriptHelper(scriptService);
            var res = new List<SummInfo>();
            foreach (DataRow dataRow in table.Rows)
            {
               var nuName = dataRow["NUNAME"].ToString().TrimEnd();
               var summ = (double)dataRow["SUMM"];
               var ist = dataRow["ISTNAME"].ToString().TrimEnd();
               if (nuName == "Перенос задолженности" || Math.Abs(summ) < double.Epsilon || ist == "Бюджетное финансирование")
                  continue;

               var flCode = dataRow["FLCode"].ToString().MyToInt();
               var dolgCode = dataRow["DolgCode"].ToString().MyToInt();
               var fio = dataRow["Fio"].ToString().TrimEnd();
               var snils = dataRow["SNILS"].ToString().TrimEnd();
               var mPrin = dataRow["MPRIN"].ToString().ToKDate().IsValid()
                   ? dataRow["MPRIN"].ToString().ToKDate().ToAbsMonth()
                   : -1;
               var mNach = dataRow["MNACH"].ToString().ToKDate().IsValid()
                   ? dataRow["MNACH"].ToString().ToKDate().ToAbsMonth()
                   : -1;

               var istCode = script.Call<int>("DefineIstCode", ist);
               var objCode = script.Call<int>("DefineObjCode", ist, dolgCode, flCode);
               var nuCode = script.Call<int>("DefineNuCode", nuName);

               res.Add(new SummInfo()
               {
                  FLCode = flCode,
                  DolgCode = dolgCode,
                  Fio = fio,
                  Snils = snils,
                  Summ = summ,
                  IstCode = istCode,
                  ObjCode = objCode,
                  MPrin = mPrin,
                  MNach = mNach,
                  NuCode = nuCode,
                  NuName = nuName
               });
            }

            FillMainSrc(res, script);
            FillMainObj(res, script);
            return res;
         }
         catch (Exception e)
         {
            Console.WriteLine(e);
            throw;
         }

      }

      private static void FillMainObj(List<SummInfo> res, ScriptHelper script)
      {
         var flMainObj = new Dictionary<int, int>(); //ключ - код должности, значение - главный обьект
         using (var progress = new Progress((uint)res.Count, "Вычисляем итоговый объект.."))
         {
            foreach (var summInfo in res)
            {
               if (progress.Increment())
                  break;

               if (!flMainObj.ContainsKey(summInfo.DolgCode)) //для физ лица еще вычисляли главный источник 
               {
                  var flData = res.Where(p => p.DolgCode == summInfo.DolgCode).ToList();
                  CalcDolgnMainObj_Impl(flData, script, flMainObj);
               }

               flMainObj.TryGetValue(summInfo.DolgCode, out var flmainObj);
               summInfo.DolgnMainObjCode = flmainObj;
            }
         }
      }

      private static void FillMainSrc(List<SummInfo> res, ScriptHelper script)
      {
         var flMainSrc = new Dictionary<int, int>(); //ключ - код физ лица, значение - главный источник
         var flDolgnMainSrc = new Dictionary<int, int>(); //ключ - код должности, значение - главный источник
         using (var progress = new Progress((uint)res.Count, "Вычисляем итоговый источник.."))
         {
            foreach (var summInfo in res)
            {
               if (progress.Increment())
                  break;

               if (!flMainSrc.ContainsKey(summInfo.FLCode)) //для физ лица еще вычисляли главный источник 
               {
                  var flData = res.Where(p => p.FLCode == summInfo.FLCode).ToList();
                  CalcMainSrc_Impl(flData, script, flMainSrc);
               }

               if (!flDolgnMainSrc.ContainsKey(summInfo.DolgCode)) //для физ лица еще вычисляли главный источник 
               {
                  var data = res.Where(p => p.DolgCode == summInfo.DolgCode).ToList();
                  CalcDolgnMainSrc_Impl(data, script, flDolgnMainSrc);
               }

               flMainSrc.TryGetValue(summInfo.FLCode, out var flmainSrc);
               summInfo.LsMainSrc = flmainSrc;

               flDolgnMainSrc.TryGetValue(summInfo.DolgCode, out var dolgnMainSrc);
               summInfo.DolgnMainSrcCode = dolgnMainSrc;
            }
         }

      }


      private static void CalcDolgnMainObj_Impl(List<SummInfo> data, ScriptHelper script, Dictionary<int, int> flMainObj)
      {
         var dolgCode = data.First().DolgCode;
         var mainObjCode = -1;

         var objList = data.GroupBy(p => p.ObjCode).ToList();
         if (objList.Count == 1)
            mainObjCode = objList.First().Key;
         else //главный обьект тот -сумма начислений по которому максимальная
         {
            var sumMainObj = (double)-1;
            foreach (var objData in objList)
            {
               var summObj = objData.Where(p => script.Call<int>("IsCodeNach", (double)p.NuCode) == 1)
                   .Sum(p => p.Summ);

               if (!(summObj > sumMainObj))
                  continue;
               mainObjCode = objData.Key;
               sumMainObj = summObj;
            }
         }

         flMainObj.Add(dolgCode, mainObjCode);
      }
      private static void CalcDolgnMainSrc_Impl(List<SummInfo> data, ScriptHelper script, Dictionary<int, int> flDolgnMainSrc)
      {
         var dolgCode = data.First().DolgCode;
         var mainSrcCode = -1;

         var srcList = data.GroupBy(p => p.IstCode).ToList();
         if (srcList.Count == 1)
            mainSrcCode = srcList.First().Key;
         else //главный обьект тот -сумма начислений по которому максимальная
         {
            var sumMainsrc = (double)-1;
            foreach (var srcData in srcList)
            {
               var summSrc = srcData.Where(p => script.Call<int>("IsCodeNach", (double)p.NuCode) == 1)
                   .Sum(p => p.Summ);

               if (!(summSrc > sumMainsrc))
                  continue;
               mainSrcCode = srcData.Key;
               sumMainsrc = summSrc;
            }
         }

         flDolgnMainSrc.Add(dolgCode, mainSrcCode);
      }

      private static void CalcMainSrc_Impl(List<SummInfo> flData, ScriptHelper script, IDictionary<int, int> flMainSrc)
      {
         var flCode = flData.First().FLCode;
         var mainSrcCode = -1;

         var srcList = flData.GroupBy(p => p.IstCode).ToList();
         //один источник - он главный
         if (srcList.Count == 1)
            mainSrcCode = srcList.First().Key;
         else //главный источник тот -сумма начислений по которому максимальная
         {
            var sumMainSrc = (double)-1;
            foreach (var srcData in srcList)
            {
               var summSrc = srcData.Where(p => script.Call<int>("IsCodeNach", (double)p.NuCode) == 1)
                   .Sum(p => p.Summ);

               if (!(summSrc > sumMainSrc))
                  continue;
               mainSrcCode = srcData.Key;
               sumMainSrc = summSrc;
            }
         }

         flMainSrc.Add(flCode, mainSrcCode);
      }

      public static List<NdflDeductionInfo> GetNdflDeduction(DataTable table)
      {
         if (!table.Columns.Contains("FLCode") ||
             !table.Columns.Contains("Fio") ||
             !table.Columns.Contains("SNILS") ||
             !table.Columns.Contains("VICHCODE") ||
             !table.Columns.Contains("VICHCNT") ||
             !table.Columns.Contains("DB") ||
             !table.Columns.Contains("DE"))
            throw new Exception("Не удалось найти данные о суммах");

         //получаем информацию из файла
         var res = new List<LsData.LsDataHelper.NdflDeductInfo>();
         foreach (DataRow dataRow in table.Rows)
         {
            var db = dataRow["DB"].ToString().ToKDate().IsValid()
                ? dataRow["DB"].ToString().ToKDate()
                : Consts.EmptyDate;
            var de = dataRow["DE"].ToString().ToKDate().IsValid()
                ? dataRow["DE"].ToString().ToKDate()
                : Consts.EmptyDate;

            if (db > de)
               continue; //не обрабатываем недействующие вычеты

            res.Add(new LsData.LsDataHelper.NdflDeductInfo()
            {
               FLCode = dataRow["FLCode"].ToString().TrimEnd(),
               Fio = dataRow["Fio"].ToString().TrimEnd(),
               Snils = dataRow["SNILS"].ToString().TrimEnd(),
               DeductionCode = dataRow["VICHCODE"].ToString().Split('/')[0].ToInt(),
               DeductionCount = dataRow["VICHCNT"].ToString().ToInt(),
               DateBeg = db,
               DateEnd = de
            });
         }

         var result = new List<NdflDeductionInfo>();
         //группируем по физ лицу
         foreach (var flCodeGroup in res.GroupBy(p => p.FLCode))
         {
            //группируем по дате начала вычета
            foreach (var dbGroup in flCodeGroup.GroupBy(p => p.DateBeg))
            {
               var value = "";
               var firstDeduction = true;
               var deProcessed = new List<DateTime>();
               //перебираем все вычеты и формируем строку, которую нужно разнести в к.ч.
               foreach (var ndflDeductionInfo in dbGroup.OrderByDescending(p => p.DateEnd))
               {
                  if (ndflDeductionInfo.DateEnd.IsValid() && !deProcessed.Contains(ndflDeductionInfo.DateEnd)
                  ) //указана дата окончания -> нужно закрыть вычет
                  {
                     result.Add(new NdflDeductionInfo()
                     {
                        FLCode = ndflDeductionInfo.FLCode,
                        Fio = ndflDeductionInfo.Fio,
                        Snils = ndflDeductionInfo.Snils,
                        //закрываем вычет с первого числа месяца, следующего за DateEnd
                        DateBeg = new DateTime(ndflDeductionInfo.DateEnd.Year, ndflDeductionInfo.DateEnd.Month,
                             ndflDeductionInfo.DateEnd.CountDay()).AddDays(1).ToKDateString(),
                        Value = value
                     });
                     deProcessed.Add(ndflDeductionInfo.DateEnd);
                  }

                  if (!firstDeduction) value += ";";
                  else
                     firstDeduction = false;
                  value += $"{ndflDeductionInfo.DeductionCount},{ndflDeductionInfo.DeductionCode}";
               }

               result.Add(new NdflDeductionInfo()
               {
                  FLCode = flCodeGroup.Key,
                  Fio = flCodeGroup.First().Fio,
                  Snils = flCodeGroup.First().Snils,
                  //закрываем вычет с первого числа месяца, следующего за DateEnd
                  DateBeg = dbGroup.Key.ToKDateString(),
                  Value = value
               });
            }
         }

         return result;
      }

      public static List<PlanSummInfo> GetPlanSumm(DataTable table, IScriptService scriptService)
      {
         if (!table.Columns.Contains("FLCode") ||
             !table.Columns.Contains("DolgCode") ||
             !table.Columns.Contains("Fio") ||
             !table.Columns.Contains("SNILS") ||
             !table.Columns.Contains("NuName") ||
             !table.Columns.Contains("SUMM") ||
             !table.Columns.Contains("Bz"))
            throw new Exception("Не удалось найти данные о плановых начислениях");

         var script = new ScriptHelper(scriptService);
         var res = new List<PlanSummInfo>();
         foreach (DataRow dataRow in table.Rows)
         {
            var summ = (double)dataRow["SUMM"];
            var flCode = dataRow["FLCode"].ToString().MyToInt();
            var dolgCode = dataRow["DolgCode"].ToString().MyToInt();
            var fio = dataRow["Fio"].ToString().TrimEnd();
            var snils = dataRow["SNILS"].ToString().TrimEnd();
            var nuName = dataRow["NuName"].ToString().TrimEnd();
            var nuCode = script.Call<int>("DefineNuCode", nuName);
            res.Add(new PlanSummInfo()
            {
               FLCode = flCode,
               DolgCode = dolgCode,
               Fio = fio,
               NuName = nuName,
               NuCode = nuCode,
               Snils = snils,
               Summ = summ,
               Bz = (double)dataRow["Bz"]
            });
         }

         return res;
      }

      public static List<SummSrednInfo> GetSummSredn(DataTable table, IScriptService scriptService)
      {

         if (!table.Columns.Contains("FLCode") ||
             !table.Columns.Contains("DolgCode") ||
             !table.Columns.Contains("Fio") ||
             !table.Columns.Contains("SNILS") ||
             !table.Columns.Contains("ISTNAME") ||
             !table.Columns.Contains("SUMM") ||
             !table.Columns.Contains("CNTDAY") ||
             !table.Columns.Contains("MPRIN") ||
             !table.Columns.Contains("NUNAME"))
            throw new Exception("Не удалось найти данные для расчета среднего(отпуск)");

         var script = new ScriptHelper(scriptService);
         var res = new List<SummSrednInfo>();
         foreach (DataRow dataRow in table.Rows)
         {
            var summ = (double)dataRow["SUMM"];
            if (Math.Abs(summ) < double.Epsilon)
               continue;
            var flCode = dataRow["FLCode"].ToString().TrimEnd();
            var dolgCode = dataRow["DolgCode"].ToString().TrimEnd();
            var fio = dataRow["Fio"].ToString().TrimEnd();
            var snils = dataRow["SNILS"].ToString().TrimEnd();
            var mPrin = dataRow["MPRIN"].ToString().ToKDate().IsValid()
                ? dataRow["MPRIN"].ToString().ToKDate().ToAbsMonth()
                : -1;
            var ist = dataRow["ISTNAME"].ToString().TrimEnd();
            var istCode = script.Call<int>("DefineIstCode", ist);
            var objCode = script.Call<int>("DefineObjCode", ist, dolgCode, flCode);
            res.Add(new SummSrednInfo()
            {
               FLCode = flCode,
               DolgCode = dolgCode,
               Fio = fio,
               NuCode = 86,
               Snils = snils,
               Summ = summ,
               Days = (double)dataRow["CNTDAY"],
               MPrin = mPrin,
               NuName = dataRow["NUNAME"].ToString().TrimEnd(),
               IstCode = istCode,
               ObjCode = objCode
            });
         }
         return res;
      }


      public static List<ExcludePeriodInfo> GetExcludePeriods(DataTable data)
      {
         if (!data.Columns.Contains("FLCode") ||
             !data.Columns.Contains("DolgCode") ||
             !data.Columns.Contains("Fio") ||
             !data.Columns.Contains("SNILS") ||
             !data.Columns.Contains("PERIODTYPE") ||
             !data.Columns.Contains("PERIODNAME") ||
             !data.Columns.Contains("DATEB") ||
             !data.Columns.Contains("DATEE"))
            throw new Exception("Не удалось найти данные c исключаемыми периодами");
         var res = new List<ExcludePeriodInfo>();
         foreach (DataRow dataRow in data.Rows)
         {
            res.Add(new ExcludePeriodInfo()
            {
               FLCode = dataRow["FLCode"].ToString().TrimEnd(),
               DolgCode = dataRow["DolgCode"].ToString().TrimEnd(),
               Fio = dataRow["Fio"].ToString().TrimEnd(),
               Snils = dataRow["SNILS"].ToString().TrimEnd(),
               PeriodType = dataRow["PERIODTYPE"].ToString().TrimEnd(),
               PeriodName = dataRow["PERIODNAME"].ToString().TrimEnd(),
               DateBegin = dataRow["DATEB"].ToString().TrimEnd(),
               DateEnd = dataRow["DATEE"].ToString().TrimEnd()
            });
         }
         return res;
      }
   }
}