﻿using System;
using System.Collections.Generic;
using System.Linq;
using SKBKontur.KZ.Common;
using SKBKontur.KZ.Core;
using SKBKontur.KZ.User.LsData;
using XMachineLib;

namespace SKBKontur.KZ.User.Helpers
{
   public class TnHelper
   {
      private readonly Dictionary<string, int> FlCode_Tn;
      private readonly List<int> ReserverdTn;
      public ILsList LsList { get; }
      public IXMachine XMachine { get; }
      public IOptions Options { get; }

      public TnHelper(ILsList lsList,IXMachine xMachine,IOptions options)
      {
         LsList = lsList;
         XMachine = xMachine;
         Options = options;
         FlCode_Tn = new Dictionary<string, int>();
         ReserverdTn = new List<int>();
      }

      public void FillFromLsList()
      {
         foreach (var lsListElem in LsList.WithReadAccess())
         {
            using (var ls=LsList.ReadLS(lsListElem.Id,LSReadFlags.GlobalLS|LSReadFlags.ReadOnly))
            {
               var flCode = ls.Kch().GetValue(Options.KchNumFromPseudo("FLCode")).Value.TrimEnd();
               //не указан код
               if(flCode.Length<1)
                  continue;

               if(FlCode_Tn.ContainsKey(flCode))
                  continue;

               FlCode_Tn.Add(flCode,lsListElem.Tn);
            }
         }
      }

      public int CalcTn(string flCode, string dolgnCode, List<LsInfo> data)
      {
         if (FlCode_Tn.ContainsKey(flCode))
            return FlCode_Tn[flCode];

         var tn = CalcTnImpl(flCode, dolgnCode, data);
         
         FlCode_Tn.Add(flCode,tn);
         return tn;
      }

      private int CalcTnImpl(string flCode, string dolgnCode, List<LsInfo> data)
      {
         //все должности физ.лица
         var dolgnList = data.Where(p => p.FLCode == flCode).ToList();

         //Если исп. должность одна - возвращаем код этой исп. должности
         if (dolgnList.Count == 1)
            return TnByDolgnCode(dolgnList[0].DolgCode);

         //Иначе возвращаем код той исп.должности, у которойVidDolgn==Основное место работы
         var osnDolg = dolgnList.FirstOrDefault(p => p.IsMain);
         if (osnDolg != null)
            return TnByDolgnCode(osnDolg.DolgCode);

         var noSovmDolg = dolgnList.FirstOrDefault(p => !p.IsSovm);
         if(noSovmDolg!=null)
            return TnByDolgnCode(noSovmDolg.DolgCode);


         return TnByDolgnCode(dolgnList.First().DolgCode);
         //!!!Оказывается, есть внешние совместители, у которых есть внутренне сов-во

         throw new Exception($"Не удалось определить ТН для FlCode:{flCode} и DolgnCode:{dolgnCode} ");
         
      }

      private int TnByDolgnCode(string dolgCode)
      {
         if(dolgCode.IsDigitsOnly() && IsFree(dolgCode.Atoi()))
            return dolgCode.Atoi();

         //присваиваем новый ТН... первый "свободный"
         var firstTnForNewLs = (int)XMachine.Evaluate("return FirstTnForNewLs;");
         var maxTnLsList = LsList.Full.Count>0? LsList.Full.Max(p => p.Tn):0;
         var maxUsedTn = ReserverdTn.Count>0? ReserverdTn.Max():0;

         var newTn = Math.Max(Math.Max(firstTnForNewLs, maxTnLsList), maxUsedTn)+1;

         ReserverdTn.Add(newTn);
         return newTn;
      }

      private bool IsFree(int tn)
      {
         return FlCode_Tn.Values.All(p => p != tn)
            && LsList.Full.All(p=>p.Tn!=tn);
      }
   }
}
