﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using SKBKontur.KZ.Common;
using SKBKontur.KZ.Core;
using SKBKontur.KZ.Forms;
using SKBKontur.KZ.OfficeInterop;
using SKBKontur.KZ.User.Helpers;
using SKBKontur.KZ.User.LsData;
using XMachineLib;
using LsDataHelper = SKBKontur.KZ.User.Helpers.LsDataHelper;

namespace SKBKontur.KZ.User
{
    public partial class UserClass
    {
        public string GetFileName(string initDir, string ext)
        {
            var fileDialog = new OpenFileDialog();
            if (!string.IsNullOrEmpty(initDir))
                fileDialog.InitialDirectory = initDir;
            if (!string.IsNullOrEmpty(ext))
                fileDialog.Filter = ext;
            return fileDialog.ShowDialog() == DialogResult.OK ? fileDialog.FileName : "";
        }

        #region ImportLs_Impl

        public ArrayList GetDepartments(string filePath)
        {
            var dir = Path.GetDirectoryName(filePath);
            var fileName = Path.GetFileNameWithoutExtension(filePath);
            var data = DbfHelper.GetDbfData(dir, fileName);

            var departments = GetDepartmentsImpl(data);
            var departmentList = new ArrayList(departments.Count);
            foreach (var department in departments)
            {
                // departmentList.Add(department); 
                departmentList.Add(string.Format("{0};{1}", department.Key, department.Value));
            }
            return departmentList;
        }
        public bool LsExist(int tn)
        {
            return LsDataHelper.LsExist(KzApp.GetService<ILsList>(), tn);
        }
        public bool CreateLs(int tn, int departmentCode)
        {
            return LsDataHelper.CreateLs(KzApp, tn, departmentCode);
        }
        private Dictionary<int, string> GetDepartmentsImpl(DataTable table)
        {
            var res = new Dictionary<int, string>();
            if (!table.Columns.Contains("PodrCode") ||
                !table.Columns.Contains("PodrName"))
                throw new Exception("Не удалось найти данные о подразделениях");
            foreach (DataRow dataRow in table.Rows)
            {
                var departmentCode = dataRow["PodrCode"].ToString().MyToInt();
                if (res.ContainsKey(departmentCode))
                    continue;
                var departmentName = dataRow["PodrName"].ToString().TrimEnd();
                res.Add(departmentCode, departmentName);
            }
            return res;
        }
        public ArrayList GetLsDataArrayList(string filePath)
        {
            try
            {
                var dir = Path.GetDirectoryName(filePath);
                var fileName = Path.GetFileNameWithoutExtension(filePath);
                var data = DbfHelper.GetDbfData(dir, fileName);

                var tnHelper = new TnHelper(KzApp.GetService<ILsList>(), KzApp.GetService<IScriptService>().GetXMachine(), KzApp.GetService<IOptions>());
                
                var lsDataList = LsDataHelper.GetLsData(data, tnHelper);
                //проверим уникальность код должности.
             /*   if (!CheckUniqueDolgnCode(lsDataList))
                    return null;*/
                var lsArrayList = new ArrayList(lsDataList.Count);
                foreach (var lsInfo in lsDataList)
                {
                    lsArrayList.Add((object)lsInfo);
                }
                return lsArrayList;
            }
            catch (Exception e)
            {
                MsgBox.ShowError(e.Message);
                Logger.Error(e);
                return null;
            }

        }
        private bool CheckUniqueDolgnCode(List<LsInfo> lsDataList)
        {
            var dubliInfo = new Dictionary<string, List<string>>();
            foreach (var dolgnInfo in lsDataList.OrderBy(p => p.DolgCode))
            {
                var key = $"{dolgnInfo.F} {dolgnInfo.I} {dolgnInfo.O} (снилс:{dolgnInfo.Snils})";
                if (!dubliInfo.ContainsKey(dolgnInfo.DolgCode))
                    dubliInfo.Add(dolgnInfo.DolgCode, new List<string>() { key });
                else
                {
                    dubliInfo.TryGetValue(dolgnInfo.DolgCode, out var keyList);
                    if (!keyList.Contains(key))
                        keyList.Add(key);
                }
            }
            var dubli = dubliInfo.Where(p => p.Value.Count > 1).ToList();
            if (dubli.Count < 1)
                return true;
            PrintDubli(dubli);
            MsgBox.ShowInfo($"количество должностей с дублями {dubli.Count}");
            return false;
        }

        private void PrintDubli(List<KeyValuePair<string, List<string>>> dubli)
        {
            var office = new OfficeService { PreferredOffice = KzApp.GetService<IOptions>().PreferredOffice() };
            using (var wb = office.CreateWorkbook()) //создали книгу
            {
                try
                {
                    while (wb.WorksheetsCount > 1)
                        wb.DeleteWorksheet(1);
                    var row = 1;
                    var wsh = wb.ActiveWorksheet;
                    wsh.Row(row).Font.Bold = true;
                    wsh.Row(row).Font.Color = Color.Red.ToArgb();
                    wsh.Cell(row++, 1).Value = $"Код должности не уникален!";

                    using (var progress = new Progress((uint)dubli.Count, "Формируется отчет.."))
                    {
                        foreach (var keyValuePair in dubli)
                        {
                            wsh.Row(row).Font.Bold = true;
                            wsh.Cell(row++, 1).Value = $"Код должности: {keyValuePair.Key}";

                            wsh.Cell(row, 1).Value = "Список сотрудников:";
                            foreach (var key in keyValuePair.Value)
                            {
                                wsh.Cell(row++, 2).Value = key;
                            }
                            if(progress.Increment())
                                break;
                        }
                    }

           
                    wsh.Column(1).AutoFitWidth();
                    wsh.Column(2).AutoFitWidth();
                }
                catch (Exception)
                {
                    MsgBox.ShowError("Ошибка при печати квитка");
                }
                finally
                {
                    wb.GetWorksheet(1).Activate();
                    wb.Visible = true;
                    wb.Detach();
                }
            }
        }

        #endregion

        #region FillKch_Impl
        public ArrayList GetKchDataList(string filePath)
        {
            try
            {
                var dir = Path.GetDirectoryName(filePath);
                var fileName = Path.GetFileNameWithoutExtension(filePath);
                var data = DbfHelper.GetDbfData(dir, fileName);

                var kchDataList = LsDataHelper.GetKchData(data, KzApp.GetService<IScriptService>());
                var lsArrayList = new ArrayList(kchDataList.Count);
                foreach (var lsInfo in kchDataList)
                {
                    lsArrayList.Add(lsInfo);
                }
                return lsArrayList;
            }
            catch (Exception e)
            {
                MsgBox.ShowError(e.Message);
                Logger.Error(e);
                return null;
            }
        }

        #endregion

        #region ImportTabel_Impl
        public ArrayList GetTabelData(string filePath)
        {
            try
            {
                var dir = Path.GetDirectoryName(filePath);
                var fileName = Path.GetFileNameWithoutExtension(filePath);
                var data = DbfHelper.GetDbfData(dir, fileName);

                var tabelData = LsDataHelper.GetTabelData(data);
                var lsArrayList = new ArrayList(tabelData.Count);
                foreach (var lsInfo in tabelData.OrderBy(p=>p.FLCode)) //сортируем по коду физ лица для оптимизации загрузки
                {
                    lsArrayList.Add((object)lsInfo);
                }
                return lsArrayList;
            }
            catch (Exception e)
            {
                MsgBox.ShowError(e.Message);
                Logger.Error(e);
                return null;
            }
        }

        #endregion
        #region ImportSumm_Impl
        public ArrayList GetSummData(string filePath)
        {
            try
            {
                var dir = Path.GetDirectoryName(filePath);
                var fileName = Path.GetFileNameWithoutExtension(filePath);
                var data = DbfHelper.GetDbfData(dir, fileName);

                var summData = LsDataHelper.GetSummData(data,KzApp.GetService<IScriptService>());
                if (summData.Any(p =>p.IstCode == -1 || p.ObjCode == -1 || p.NuCode == -1 || p.MNach == -1 || p.MPrin == -1)) //for test
                {
                    MsgBox.ShowError("В файле некорректные данные");
                    var dataList = summData.Where(p =>p.IstCode == -1 || p.ObjCode == -1 || p.NuCode == -1 || p.MNach == -1 || p.MPrin == -1);
                    BadDataToLog(dataList);
                    return null;
                }
                    

                var lsArrayList = new ArrayList(summData.Count);
                foreach (var lsInfo in summData.OrderBy(p => p.FLCode)) //сортируем по коду физ лица для оптимизации загрузки
                {
                    lsArrayList.Add(lsInfo);
                }
                return lsArrayList;
            }
            catch (Exception e)
            {
                MsgBox.ShowError(e.Message);
                Logger.Error(e);
                return null;
            }
        }

        private void BadDataToLog(IEnumerable<SummInfo> dataList)
        {
            foreach (var d in dataList)
            {
                Logger.Warn($"FLCode:{d.FLCode} DolgCode:{d.DolgCode} Fio:{d.Fio} IstCode:{d.IstCode} ObjCode:{d.ObjCode} NuCode:{d.NuCode} MNach:{d.MNach} MPrin:{d.MPrin}  Summ:{d.Summ}");
            }
            
        }

        #endregion
        public ArrayList GetPlanSumm(string filePath)
        {
            try
            {
                var dir = Path.GetDirectoryName(filePath);
                var fileName = Path.GetFileNameWithoutExtension(filePath);
                var data = DbfHelper.GetDbfData(dir, fileName);

                var summData = LsDataHelper.GetPlanSumm(data, KzApp.GetService<IScriptService>());
                var lsArrayList = new ArrayList(summData.Count);
                foreach (var lsInfo in summData.OrderBy(p => p.FLCode)) //сортируем по коду физ лица для оптимизации загрузки
                {
                    lsArrayList.Add(lsInfo);
                }
                return lsArrayList;
            }
            catch (Exception e)
            {
                MsgBox.ShowError(e.Message);
                Logger.Error(e);
                return null;
            }
        }
        public ArrayList GetSummSrednInfo(string filePath)
        {
            try
            {
                var dir = Path.GetDirectoryName(filePath);
                var fileName = Path.GetFileNameWithoutExtension(filePath);
                var data = DbfHelper.GetDbfData(dir, fileName);

                var summSrednInfo = LsDataHelper.GetSummSredn(data, KzApp.GetService<IScriptService>());
                var lsArrayList = new ArrayList(summSrednInfo.Count);
                foreach (var lsInfo in summSrednInfo.OrderBy(p => p.FLCode)) //сортируем по коду физ лица для оптимизации загрузки
                {
                    lsArrayList.Add(lsInfo);
                }
                return lsArrayList;
            }
            catch (Exception e)
            {
                MsgBox.ShowError(e.Message);
                Logger.Error(e);
                return null;
            }
        }
        public ArrayList GetNdflDeduction(string filePath)
        {
            try
            {
                var dir = Path.GetDirectoryName(filePath);
                var fileName = Path.GetFileNameWithoutExtension(filePath);
                var data = DbfHelper.GetDbfData(dir, fileName);

                var deductions = LsDataHelper.GetNdflDeduction(data);
                var lsArrayList = new ArrayList(deductions.Count);
                foreach (var lsInfo in deductions.OrderBy(p => p.FLCode)) //сортируем по коду физ лица для оптимизации загрузки
                {
                    lsArrayList.Add(lsInfo);
                }
                return lsArrayList;

            }
            catch (Exception e)
            {
                MsgBox.ShowError(e.Message);
                Logger.Error(e);
                return null;
            }
        }
       /* public void CheckDubli(string filePath)
        {
            var dir = Path.GetDirectoryName(filePath);
            var fileName = Path.GetFileNameWithoutExtension(filePath);
            var data = DbfHelper.GetDbfData(dir, fileName);

            var kchDataList = LsDataHelper.GetLsData(data);

            var dubliInfo = new Dictionary<int, List<string>>();
            foreach (var dolgnInfo in kchDataList.OrderBy(p => p.DolgCode))
            {
                var fio = $"{dolgnInfo.F} {dolgnInfo.I} {dolgnInfo.O}";
                if (!dubliInfo.ContainsKey(dolgnInfo.DolgCode))
                    dubliInfo.Add(dolgnInfo.DolgCode, new List<string>() { fio });
                else
                {
                    dubliInfo.TryGetValue(dolgnInfo.DolgCode, out var fioList);
                    if (!fioList.Contains(fio))
                        fioList.Add(fio);
                }
            }

            int count = 0;
            foreach (var pair in dubliInfo.Where(p => p.Value.Count > 1))
            {
                Logger.Debug($"DolgCode:{pair.Key} - количество дублей: {pair.Value.Count}");
                foreach (var s in pair.Value)
                {
                    Logger.Debug(s);
                }
                count++;
            }

            MsgBox.ShowInfo($"количество должностей с дублями {count}");
        }*/
        public ArrayList GetExcludePeriodArrayList(string filePath)
        {
            try
            {
                var dir = Path.GetDirectoryName(filePath);
                var fileName = Path.GetFileNameWithoutExtension(filePath);
                var data = DbfHelper.GetDbfData(dir, fileName);
                //сортируем по физ лицам и должностям
                var excludePeriods = LsDataHelper.GetExcludePeriods(data).OrderBy(p=>$"{p.FLCode}_{p.DolgCode}").ToList();

                var excludePeriodArrayList = new ArrayList(excludePeriods.Count);
                foreach (var excludePeriod in excludePeriods)
                {
                    excludePeriodArrayList.Add(excludePeriod);
                }
                return excludePeriodArrayList;
            }
            catch (Exception e)
            {
                MsgBox.ShowError(e.Message);
                Logger.Error(e);
                return null;
            }

        }

    }
}
